/*
	Purpose	:	This is the main file.
	Author	:	Alex Stojan
*/

#include <fstream>
#include <map>

#include "antlr/AST.hpp"
#include "SourceLexer.hpp"
#include "SourceParser.hpp"
#include "ASTParser.hpp"

#include "Globals.h"

std::map<std::string, std::string> Compiler::inputArgs;

int main(int argc, char* argv[]) {
	ANTLR_USING_NAMESPACE(std)
	ANTLR_USING_NAMESPACE(antlr)
	
	string src = argv[1];
	const string errorHdr = "??? " + src;

	for (int i = 2;  i < argc;  i += 2) {
		inputArgs[string(argv[i])] = string(argv[i + 1]);
	}

	string path = inputArgs["/path"];
	string fullName = path + src;

	ifstream source(fullName.c_str());

	try {
		SourceLexer lexer(source);
		
		try {
			SourceParser parser(lexer);

			// make an ast factory
			ASTFactory ast_factory("ASTNodeExt", ASTNodeExt::factory);

			// initialize and put it in the parser...
			parser.initializeASTFactory(ast_factory);
			parser.setASTFactory(&ast_factory);

			// Parse the input expression
			parser.start();
			RefASTNodeExt t = RefASTNodeExt(parser.getAST());

			if (inputArgs["/ast"] == "y") {
				cout << "\n*** Parser Output ***\n\n" << t->toStringTree() << endl;
			}

			if (t) {
				ASTParser astParser;
				astParser.start(t);
			}
			else
				cerr << errorHdr << endl;

		}
		catch (Compiler::CompilerException& e) {
			cerr << errorHdr << e.what() << endl;
		}
		catch (ANTLRException&) {
			cerr << errorHdr << " (" << lexer.getLine() << "): Syntax error" << endl;
		}
		catch (...) {
			cerr << errorHdr << ": Error" << endl;
		}
	}
	catch (...) {
		cerr << errorHdr << ": Error" << endl;
	}

	return 0;
}
