#ifndef _GLOBALS_
#define _GLOBALS_

#include <string>

#include "IC.h"

typedef int INT32;
typedef float FLOAT32;
typedef char CHAR8;

typedef enum {
	OK,
	OUT_OF_MEMORY
} VM_STATUS_CODE;

typedef enum {
	SUCCESS,
	INVALID_SYS_CODE
} SYS_CALL_RESULT;

typedef enum {
	STRING,
	CHAR,
	BOOL,
	INT,
	FLOAT
} TYPE_CODE;

typedef std::pair<INT32, INT32*> SysCallPair;

const INT32 WORD_SIZE			= 4;

const INT32 SYSCODE_WRITE		= 1;
const INT32 SYSCODE_READ		= 2;
const INT32 SYSCODE_WRITELN	= 3;

const int MAX_ASM_LINE = 80;

struct Instruction {
	Instruction(): opcode(-1), optr1(-1), optr2(-1) {}

	std::string mnemonic;
	int opcode;
	int optr1;
	int optr2;
};

struct DataItem {
	enum ItemType { STRING, FLOAT, BLOCK };
	ItemType type;

	DataItem(std::string s): str(s), type(STRING) {}
	DataItem(float v): floatVal(v), type(FLOAT) {}
	DataItem(int b): block(b), type(BLOCK) {}

	std::string	str;
	float			floatVal;
	int			block;
};

#endif
