/*
	Purpose	:	Opcodes and assembly tags for the interpreter (VM); currently,
					this file is shared between the compiler and the VM.
					
	Author	:	Alex Stojan
*/

#ifndef _IC_
#define _IC_

char* const DATA_SEGMENT	=	".DATA";
char* const CODE_SEGMENT	=	".CODE";
char* const FLOAT_CONST		=	".float";
char* const BLOCK_DEF		=	".block";
char* const ASCIIZ_DEF		=	".asciiz";
char* const ASM_END			=	".END";

enum OpCode {
//	OPCODE				# OF PARAMETERS
//	------				---------------
	IC_NOP,				// 0
	IC_BSTOREI,			// 0
	IC_BSTOREIZ,		// 0
	IC_WSTOREI,			// 0
	IC_WSTOREIZ,		// 0
	IC_IADD,				// 0
	IC_ISUB,				// 0
	IC_IMUL,				// 0
	IC_IDIV,				// 0
	IC_MOD,				// 0
	IC_OR,				// 0
	IC_AND,				// 0
	IC_ICMPLT,			// 0
	IC_ICMPGT,			// 0
	IC_ICMPLE,			// 0
	IC_ICMPGE,			// 0
	IC_IFFALSE,			// 0
	IC_GOTO,				// 0
	IC_INEG,				// 0
	IC_NOT,				// 0
	IC_WCMPEQ,			// 0
	IC_ITOF,				// 0
	IC_FADD,				// 0
	IC_FSUB,				// 0
	IC_FMUL,				// 0
	IC_FDIV,				// 0
	IC_FCMPLT,			// 0
	IC_FCMPGT,			// 0
	IC_FCMPLE,			// 0
	IC_FCMPGE,			// 0
	IC_FNEG,				// 0
	IC_WCMPNEQ,			// 0
	IC_ICLOAD,			// 1
	IC_INC1,				// 1
	IC_INCI,				// 0
	IC_DEC1,				// 1
	IC_DECI,				// 0
	IC_WLOAD,			// 1
	IC_BLOAD,			// 1
	IC_WLOADI,			// 0
	IC_WLOADIZ,			// 0
	IC_WSTORE,			// 1
	IC_BSTORE,			// 1
	IC_BLOADI,			// 0
	IC_BLOADIZ,			// 0
	IC_FTOI,				//	0
	IC_IFTRUE,			// 1
	IC_INCX,				// 1
	IC_DECX,				// 1
	IC_SPBLOAD,			// 1
	IC_SPWLOAD,			// 1
	IC_SPBSTORE,		// 1
	IC_SPWSTORE,		// 1
	IC_SPINC1,			// 1
	IC_SPDEC1,			// 1
	IC_GOTOI,			// 0
	IC_PUSHX,			// 0
	IC_POPXSP,			// 0
	IC_LOADSP,			// 0
	IC_SETSP,			// 1
	IC_SYSCALL,			// 2
	_END
};

const std::string mnemonic[] = {
	"nop",
	"bstorei",
	"bstoreiz",
	"wstorei",
	"wstoreiz",
	"iadd",
	"isub",
	"imul",
	"idiv",
	"mod",
	"or",
	"and",
	"icmplt",
	"icmpgt",
	"icmple",
	"icmpge",
	"iffalse",
	"goto",
	"ineg",
	"not",
	"wcmpeq",
	"itof",
	"fadd",
	"fsub",
	"fmul",
	"fdiv",
	"fcmplt",
	"fcmpgt",
	"fcmple",
	"fcmpge",
	"fneg",
	"wcmpneq",
	"icload",
	"inc1",
	"inci",
	"dec1",
	"deci",
	"wload",
	"bload",
	"wloadi",
	"wloadiz",
	"wstore",
	"bstore",
	"bloadi",
	"bloadiz",
	"ftoi",
	"iftrue",
	"incx",
	"decx",
	"spbload",
	"spwload",
	"spbstore",
	"spwstore",
	"spinc1",
	"spdec1",
	"gotoi",
	"pushx",
	"popxsp",
	"loadsp",
	"setsp",
	"syscall",
	""
};

#endif
