/*
	Purpose	:	Some general compiling-related utilities.

	Author	:	Alex Stojan
*/

#ifndef _UTILITIES_
#define _UTILITIES_

#include <boost/lexical_cast.hpp>

#include <sstream>
#include <string>

#include "Globals.h"

namespace Compiler {
	int getTypeSizeFromTypeName(std::string, SizeMetric = BYTE);
	bool isPrimitiveTypeAssignable(TypeCode, TypeCode);

	class CompilerException {
	public:
		explicit CompilerException(int lineNo, std::string s):
			line(lineNo),
			text(s) {}

		std::string what() const {
			// in some cases we might get 0 for line number (if a concrete node
			// cannot be found)
			return line > 0 ? " (" + boost::lexical_cast<std::string>(line) + "): " + text : ": " + text;
		}

	private:
		int line;
		std::string text;
	};
}

#endif
